<?php
/**
 * A class holding general data about the application and some methods for
 * the management including the entry point.
 *
 * E.g.:
 *
 * index.php?app=my_app
 *
 * You need to create a folder in the folder "application" named "my_app"
 * (without the quotes) and create a include file called
 * class_ApplicationHelper.php. You have to write the same class for your
 * application and implement the same interface called ManageableApplication
 * because this class include file will be searched for.
 *
 * It is good when you avoid more GET parameters to keep URLs short and sweet.
 * But sometimes you need some GET paramerers e.g. for your imprint or info page
 * or other linked pages which you have to create and state some informations.
 *
 * Please remember that this include file is being loaded *before* the class
 * loader is loading classes from "exceptions", "interfaces" and "main"!
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 Ship-Simu Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ApplicationHelper extends BaseApplication implements ManageableApplication, Registerable {
	/**
	 * The version number of this application
	 */
	private $appVersion = "";

	/**
	 * The human-readable name for this application
	 */
	private $appName = "";

	/**
	 * The short uni*-like name for this application
	 */
	private $shortName = "";

	/**
	 * An instance of a controller
	 */
	private $controllerInstance = null;

	/**
	 * An instance of this class
	 */
	private static $selfInstance = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Getter for an instance of this class
	 *
	 * @return	$selfInstance	An instance of this class
	 */
	public final static function getInstance () {
		// Is the instance there?
		if (is_null(self::$selfInstance)) {
			self::$selfInstance = new ApplicationHelper();
		} // END - if

		// Return the instance
		return self::$selfInstance;
	}

	/**
	 * Getter for the version number
	 *
	 * @return	$appVersion		The application's version number
	 */
	public final function getAppVersion () {
		return $this->appVersion;
	}

	/**
	 * Setter for the version number
	 *
	 * @param	$appVersion		The application's version number
	 * @return	void
	 */
	public final function setAppVersion ($appVersion) {
		// Cast and set it
		$appVersion = (string) $appVersion;
		$this->appVersion = $appVersion;
	}

	/**
	 * Getter for human-readable name
	 *
	 * @return	$appName	The application's human-readable name
	 */
	public final function getAppName () {
		return $this->appName;
	}

	/**
	 * Setter for human-readable name
	 *
	 * @param	$appName	The application's human-readable name
	 * @return	void
	 */
	public final function setAppName ($appName) {
		// Cast and set it
		$appName = (string) $appName;
		$this->appName = $appName;
	}

	/**
	 * Getter for short uni*-like name
	 *
	 * @return	$shortName	The application's short uni*-like name
	 */
	public final function getAppShortName () {
		return $this->shortName;
	}

	/**
	 * Setter for short uni*-like name
	 *
	 * @param	$shortName	The application's short uni*-like name
	 * @return	void
	 */
	public final function setAppShortName ($shortName) {
		// Cast and set it
		$shortName = (string) $shortName;
		$this->shortName = $shortName;
	}

	/**
	 * Builds the master template's name
	 *
	 * @return	$masterTemplateName		Name of the master template
	 */
	public function buildMasterTemplateName () {
		// Get short name and add suffix
		$masterTemplateName = str_replace('-', '', $this->getAppShortName()) . '_main';

		// Return it
		return $masterTemplateName;
	}

	/**
	 * Launches the admin area
	 *
	 * @return	void
	 */
	public final function entryPoint () {
		// Set this application in registry
		Registry::getRegistry()->addInstance('app', $this);

		// Create a new request object
		$requestInstance = ObjectFactory::createObjectByName('HttpRequest');

		// Default response is HTTP (HTML page) and type is 'Web'
		$response = 'http';
		$responseType = 'web';

		// Do we have another response?
		if ($requestInstance->isRequestElementSet('request')) {
			// Then use it
			$response = strtolower($requestInstance->getRequestElement('request'));
			$responseType = $response;
		} // END - if

		// ... and a new response object
		$responseClass = sprintf("%sResponse", $this->convertToClassName($response));
		$responseInstance = ObjectFactory::createObjectByName($responseClass, array($this));

		// Remember both in this application
		$this->setRequestInstance($requestInstance);
		$this->setResponseInstance($responseInstance);

		// Get the parameter from the request
		$commandName = $requestInstance->getRequestElement('page');

		// If it is null then get default command
		if (is_null($commandName)) {
			// Get default command
			$commandName = $responseInstance->getDefaultCommand();

			// Set it in request
			$requestInstance->setRequestElement('page', $commandName);
		} // END - if

		// Get a resolver
		$resolverClass = sprintf("%sControllerResolver", $this->convertToClassName($responseType));
		$resolverInstance = ObjectFactory::createObjectByName($resolverClass, array($commandName, $this));

		// Get a controller instance as well
		$this->controllerInstance = $resolverInstance->resolveController();

		// Get a web output class
		$outputInstance = ObjectFactory::createObjectByConfiguredName('output_class', array($this));

		// Set it in this application
		$this->setWebOutputInstance($outputInstance);

		// Handle the request
		$this->controllerInstance->handleRequest($requestInstance, $responseInstance);
	}

	/**
	 * Handle the indexed array of fatal messages and puts them out in an
	 * acceptable fasion
	 *
	 * @param	$messageList	An array of fatal messages
	 * @return	void
	 */
	public function handleFatalMessages (array $messageList) {
		// Walk through all messages
		foreach ($messageList as $message) {
			print("MSG:" . $message . "<br />\n");
		} // END - if
	}

	/**
	 * Assigns application-depending data
	 *
	 * @param	$templateInstance	An instance of a template engine
	 * @return	void
	 */
	public function assignExtraTemplateData (CompileableTemplate $templateInstance) {
		// Assign charset
		$templateInstance->assignConfigVariable('header_charset');
	}
}

// [EOF]
?>
