<?php
/**
 * A command resolver for local (non-hubbed) web commands including the failed government request
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 Ship-Simu Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class WebCompanyCommandResolver extends BaseCommandResolver implements CommandResolver {
	/**
	 * Last successfull resolved command
	 */
	private $lastCommandInstance = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set prefix to "Web"
		$this->setCommandPrefix('Web');
	}

	/**
	 * Creates an instance of a Web command resolver with a given default command
	 *
	 * @param	$commandName				The default command we shall execute
	 * @param	$appInstance				An instance of a manageable application helper class
	 * @return	$resolverInstance			The prepared command resolver instance
	 * @throws	EmptyVariableException		Thrown if default command is not set
	 * @throws	InvalidInterfaceException	Thrown if command does not implement interface Commandable
	 */
	public static final function createWebCompanyCommandResolver ($commandName, ManageableApplication $appInstance) {
		// Create the new instance
		$resolverInstance = new WebCompanyCommandResolver();

		// Get request instance
		$requestInstance = $appInstance->getRequestInstance();

		// Is the variable $commandName set and the command is valid?
		if (empty($commandName)) {
			// Then thrown an exception here
			throw new EmptyVariableException(array($resolverInstance, 'commandName'), self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!$resolverInstance->resolveCommandByRequest($requestInstance) instanceof Commandable) {
			// Invalid command found (missing interface?)
			throw new InvalidInterfaceException(array($userInstance, 'ManageableMember'), self::EXCEPTION_REQUIRED_INTERFACE_MISSING);
		}

		// Set the application instance
		$resolverInstance->setApplicationInstance($appInstance);

		// Return the prepared instance
		return $resolverInstance;
	}

	/**
	 * Returns an command instance for a given request class or null if
	 * it was not found
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @return	$commandInstance	An instance of the resolved command
	 * @throws	InvalidCommandException				Thrown if $commandName is
	 *												invalid
	 * @throws	InvalidCommandInstanceException		Thrown if $commandInstance
	 *												is an invalid instance
	 */
	public function resolveCommandByRequest (Requestable $requestInstance) {
		// Init instance
		$commandInstance = null;

		// This goes fine so let's resolv the command
		$commandName = str_replace('-', '_', $requestInstance->getRequestElement('app')) . '_' . $requestInstance->getRequestElement('page');

		// Is there a "failed" request?
		if ($requestInstance->isRequestElementSet('failed')) {
			// Then include with within the command name
			$commandName = $commandName . '_' . $requestInstance->getRequestElement('failed');
		} // END - if

		// Is the command empty? Then fall back to default command
		if (empty($commandName)) $commandName = $this->getConfigInstance()->getConfigEntry('default_web_command');

		// Check if command is valid
		if ($this->isCommandValid($commandName) === false) {
			// This command is invalid!
			throw new InvalidCommandException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		} // END - if

		// Get the command
		$commandInstance = $this->loadCommand($commandName);

		// And validate it
		if ((!is_object($commandInstance)) || (!$commandInstance instanceof Commandable)) {
			// This command has an invalid instance!
			throw new InvalidCommandInstanceException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		} // END - if

		// Set last command
		$this->lastCommandInstance = $commandInstance;

		// Return the resolved command instance
		return $commandInstance;
	}

	/**
	 * Resolves the command by its direct name and returns an instance of its class
	 *
	 * @param	$commandName		The direct command name we shall resolve
	 * @return	$commandInstance	An instance of the command class
	 * @throws	InvalidCommandException		Thrown if $commandName is invalid
	 */
	public function resolveCommand ($commandName) {
		// Initiate the instance variable
		$commandInstance = null;

		// Is the command empty? Then fall back to default command
		if (empty($commandName)) $commandName = $this->getConfigInstance()->getConfigEntry('default_web_command');

		// Check if command is valid
		if ($this->isCommandValid($commandName) === false) {
			// This command is invalid!
			throw new InvalidCommandException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		}

		// Get the command
		$commandInstance = $this->loadCommand($commandName);

		// Return the instance
		return $commandInstance;
	}

	/**
	 * "Loads" a given command and instances it if not yet cached
	 *
	 * @param	$commandName				A command name we shall look for
	 * @return	$commandInstance			A loaded command instance
	 * @throws	InvalidCommandException		Thrown if even the default
	 *										command class is missing (bad!)
	 */
	private function loadCommand ($commandName) {
		// Cache default command
		$defaultCommand = $this->getConfigInstance()->getConfigEntry('default_web_command');

		// Init command instance
		$commandInstance = null;

		// Get 'app' from the application
		$app = Registry::getRegistry()->getInstance('application')->getRequestInstance()->getRequestElement('app');

		// Create command class name
		$this->setClassName(sprintf("%s%sCommand",
			$this->getCommandPrefix(),
			$this->convertToClassName($commandName)
		));

		// Is this class loaded?
		if (!class_exists($this->getClassName())) {
			// Class not found, so throw an exception
			throw new InvalidCommandException(array($this, $defaultCommand), self::EXCEPTION_INVALID_COMMAND);
		} // END - if

		// Initiate the command
		$commandInstance = ObjectFactory::createObjectByName($this->getClassName(), array($this));

		// Return the result
		return $commandInstance;
	}
}

// [EOF]
?>
